/* Copyright: David Avis 2001, avis@cs.mcgill.ca         */
/* Demo driver for ve code using lrs                 */
/* This program computes vertices of generated cubes */
/* given by H-representation                         */


/* Adaptation to DP preimage : D. Coeurjolly*/

#include <stdio.h>
#include <string.h>
#include "lrslib.h"

#define MAXCOL 4     /* maximum number of colums */

void prat_file (char name[], lrs_mp Nin, lrs_mp Din, FILE *fout)	/*reduce and print Nin/Din  */
{
  lrs_mp Nt, Dt;
  long i;
  double num,den;
  int signe;

  fprintf (fout, "%s", name);
  /* reduce fraction */
  copy (Nt, Nin);
  copy (Dt, Din);
  reduce (Nt, Dt);
  /* print out       */
  if (sign (Nin) * sign (Din) == NEG)
    signe=-1;
  else
    signe=1;

  /*fprintf (fout, "%lu", Nt[length (Nt) - 1]);*/
  num=signe*Nt[length (Nt) - 1];

  for (i = length (Nt) - 2; i >= 1; i--)
    {
      /*fprintf (fout, FORMAT, Nt[i]);*/
      num=10*num + Nt[i];
    }
  
  den=1;
  if (!(Dt[0] == 2 && Dt[1] == 1))	/* rational */
    {
      /*fprintf (fout, "/");*/
      den=Dt[length (Dt) - 1];
      /*fprintf (fout, "%lu", Dt[length (Dt) - 1]);*/
      for (i = length (Dt) - 2; i >= 1; i--)
	{	
	  /*fprintf (fout, FORMAT, Dt[i]);*/
	  den=10*den+Dt[i];
	} 
    }
   fprintf(fout,"%f ",num/den);
}


/*On ecrit dans un fichier pour Geomview*/
void  lrs_printoutput_file(lrs_dat * Q, lrs_mp_vector output, FILE *fout)
{
  long i;

  
  if (Q->hull || zero (output[0]))	/*non vertex */
    {
      i=i; 
      /*skip si pas sommets
      for (i = 0; i < Q->n; i++)
      pmp ("", output[i]);*/

    }
  else
    {				/* vertex   */
      fprintf (fout,  "");
      for (i = 1; i < Q->n; i++)
	prat_file("", output[i], output[0], fout);
      fprintf(fout,"\n");
    }
  fflush(fout);
}


void init_DP_system(lrs_dic *P, lrs_dat *Q, int  *row)
{
  long num[MAXCOL];
  long den[MAXCOL];
  int j;
  
  /*pas de den`om*/
  for(j=0;j<Q->n;j++)
    den [j] = 1;
  
 
  for(j=0;j<4;j++)
    num[j]=0;
  
  for(j=1;j<4;j++)
    {
      /*Forme alpha>=0*/
      num[j]=1;
      lrs_set_row(P,Q,*row,num,den,GE);
      (*row)++;
      num[j]=0;
    }
  
  num[0]=1;
  for(j=1;j<4;j++)
    {
      
      num[j]=-1;
      lrs_set_row(P,Q,*row,num,den,GE);
      (*row)++;
      num[j]=0;
    }
}


void add_pixel_constraints(lrs_dic *P, lrs_dat *Q, int x,int y, int  *row)
{
  long num[MAXCOL];
  long den[MAXCOL];
  int j;

  /*pas de den`om*/
  for(j=0;j<Q->n;j++)
    den [j] = 1;
  
  num[0]=-y;
  num[1]=x;
  num[2]=1;
  lrs_set_row(P,Q,*row,num,den,GE);
  (*row)++;
  num[0]=y+1;
  num[1]=-x;
  num[2]=-1;
  lrs_set_row(P,Q,*row,num,den,GE);
  (*row)++;
}

void add_voxel_constraints(lrs_dic *P, lrs_dat *Q, int x,int y,int z, int  *row)
{
  long num[MAXCOL];
  long den[MAXCOL];
  int j;

  /*pas de den`om*/
  for(j=0;j<Q->n;j++)
    den [j] = 1;
  
  num[0]=-z;
  num[1]=x;
  num[2]=y;
  num[3]=1; 
 lrs_set_row(P,Q,*row,num,den,GE);
  (*row)++;
  num[0]=z+1;
  num[1]=-x;
  num[2]=-y;
  num[3]=-1;
  lrs_set_row(P,Q,*row,num,den,GE);
  (*row)++;
}

int
main (int argc, char *argv[])

{
  lrs_dic *P;	/* structure for holding current dictionary and indices  */
  lrs_dat *Q;	/* structure for holding static problem data             */
  lrs_mp_vector output;	/* one line of output:ray,vertex,facet,linearity */
  lrs_mp_matrix Lin;    /* holds input linearities if any are found      */


  long i;
  long col;    	/* output column index for dictionary            */
  int row;
  FILE *f,*f2;
  long *count;
  char c;
  int nb_vox,x,y,z;

  /* Global initialization - done once */  
  if ( !lrs_init ("Preimage:\n"))
    return 1;
  
  lrs_ofp=stderr;

  /* compute the vertices of a set of hypercubes given by */
  /* their H-representations.                             */
  
  Q = lrs_alloc_dat ("LRS globals");
  if (Q == NULL)
    return 1;
  
  /* now flags in lrs_dat can be set */
  
    Q->n=4;           /* number of input columns         (dimension + 1 )  */
    Q->m=400;
    
    output = lrs_alloc_mp_vector (Q->n);

    P = lrs_alloc_dic (Q);   /* allocate and initialize lrs_dic      */
    if (P == NULL)
      return 1;
   

    row=1;
    init_DP_system(P,Q,&row);
    
    
    
    fscanf(stdin,"%d\n",&nb_vox);
    for(i=0;i<nb_vox;i++ )
      {
	fscanf(stdin,"%d%d%d\n",&x,&y,&z);
	add_voxel_constraints(P,Q,x,y,z,&row);
      }
    
    /* Build polyhedron: constraints and objective */ 
 
    

    /*   code from here   is borrowed from lrs_main   */ /* Pivot to a
    starting dictionary */

    if (!lrs_getfirstbasis (&P, Q, &Lin, FALSE))
      {
	fprintf(stderr,"basis failes\n");return 1;}
    
    /* There may have been column redundancy               */
    /* (although not for this example of hypercubes)       */
    
    /* If so the linearity space is obtained and redundant */
    /* columns are removed. User can access linearity space */
    /* from lrs_mp_matrix Lin dimensions nredundcol x d+1  */
  
  for (col = 0L; col < Q->nredundcol; col++)  /* print linearity space */
    lrs_printoutput (Q, Lin[col]);      /* Array Lin[][] holds the coeffs.   */
  
  /* We initiate reverse search from this dictionary       */
  /* getting new dictionaries until the search is complete */
  /* User can access each output line from output which is */
  /* a vertex/ray/facet from the lrs_mp_vector output      */


  f=fopen("temp.dat","w");
  do
    {
      for (col = 0; col <= P->d; col++)
        if (lrs_getsolution (P, Q, output, col))
          {
	    lrs_printoutput (Q, output);
	    lrs_printoutput_file (Q, output,f);
	  }
    }
    while (lrs_getnextbasis (&P, Q, FALSE));
  fclose(f);


  lrs_printtotals (P, Q);    /* print final totals */


  count = Q->count;
  fprintf (stdout, "\n%d \n%d\n",Q->n-1,count[1]);
  f2=fopen("temp.dat","r");
  
  while ((c=fgetc(f2)) != EOF )
    fputc(c,stdout);
  fclose(f2);

 
  
  /* free space : do not change order of next 3 lines! */  
  lrs_clear_mp_vector (output, Q->n);
  lrs_free_dic (P,Q);           /* deallocate lrs_dic */
  lrs_free_dat (Q);             /* deallocate lrs_dat */
  
  fprintf(stderr,"\n");
  /*lrs_close ("davdemo:");*/
  return 0;
}  /* end of main */

